
#ifndef __CANH__        /* Schutz gegen mehrfaches #include */
#define __CANH__

/***********************************************************************
 *  can.h
 *  Funktion: Header-Datei fuer das Softwaremodul "can.c".
 *
 *  Autor   : Hoppe
 *  Sprache : ANSI-C
 *
 *  ----------------------------------------------------------------
 *  Copyright     (c) 1999 PEAK-System Technik GmbH, Darmstadt.
 */


/*** Konstantendefinitionen ***/

/*** Baudratencodes = Registerwerte BTR0/BTR1 */
#define CAN_BAUD_1M     0x0014  /*   1 MBit / s */
#define CAN_BAUD_500K   0x001C  /* 500 kBit / s */
#define CAN_BAUD_250K   0x011C  /* 250 kBit / s */
#define CAN_BAUD_125K   0x031C  /* 125 kBit / s */
#define CAN_BAUD_100K   0x432F  /* 100 kBit / s */
#define CAN_BAUD_50K    0x472F  /*  50 kBit / s */
#define CAN_BAUD_20K    0x532F  /*  20 kBit / s */
#define CAN_BAUD_10K    0x672F  /*  10 kBit / s */
#define CAN_BAUD_5K     0x7F7F  /*   5 kBit / s */


/*** Maskenbits fuer Fehlerbedingungen ***/
#define CAN_ERR_OK        0x00
#define CAN_ERR_XMTFULL   0x01  /* Sendepuffer im Controller ist voll */
#define CAN_ERR_RCVEMPTY  0x02  /* keine neue Message empfangen */
#define CAN_ERR_OVERRUN   0x04  /* CAN-Controller wurde zu spaet gelesen */
#define CAN_ERR_BUSERROR  0x08  /* Busfehler: ein Errorcounter erreichte Limit */
#define CAN_ERR_BUSOFF    0x10  /* Busfehler: CAN-Controller ging 'Bus-Off' */
#define CAN_ERR_REGTEST   0x20  /* Fehler beim Registertest des CAN-Controllers */



/*** Typendefinitionen ***/

/* Eine CAN-Message */
typedef struct {
        unsigned short int ID ;      /* 11 Bit-Kennung                         */
        unsigned char      RTR ;     /* TRUE, wenn remote request              */
        unsigned char      LEN ;     /* Anzahl der gueltigen Datenbytes (0..8) */
        unsigned char      DATA[8] ; /* Datenbytes 0..7                        */
} TCANMsg ;


/*** Prototypen fuer Methoden ***/

int     CAN_Init(int _portbase, unsigned short int btr0btr1) ;
void    CAN_registerMsg(unsigned short int ID) ;
int     CAN_Read(TCANMsg *PCANMsg) ;
int     CAN_Write(TCANMsg *PCANMsg) ;
int     CAN_Status(void) ;
void    CAN_Close(void) ;

#endif /* __CANH__ */
