

/************************************************************************
 *      hwpelicn.inc
 *      Allgemeine Zugriffsprozeduren fuer einen direkt angeschlossenen
 *      SJA1000 im PeliCAN-Mode.
 *      OCRVALUE/HARDRESET/REGREAD/REGWRITE muessen extern im
 *      hardwarespezifischen Treibercode definiert worden sein;
 *      dabei heisst der Zeiger auf den Driver immer 'CANDriver'.
 *     (c) 1999 PEAK-System Technik
 *      */

#ifndef NTVERSION
#define DEBUGOUT                // Testmessages AN
#endif

/*** Bitmasken fuer IO-Konfiguration */
#if CAN_CRX1REF == 1
/*** Referenzspannung wird extern ueber CRX1 angelegt, CRX0 ist RxD ***/
#define RX0A    0x80            /* Cmd: RX0 Active = 1 */
#define RX1A    0x40            /* Cmd: RX1 Active = 1 */
#define RA      0               /* Control: Reference Active = 0 */
#else
/*** Referenz wird intern erzeugt ***/
#define RX0A    0x80            /* Cmd: RX0 Active = 1 */
#define RX1A    0x00            /* Cmd: RX1 Active = 0 */
#define RA      0x20            /* Control: Reference Active = 1 */
#endif


/**************************************************************************
 *      isinstalled()
 *      Pruefung, ob der SJA 1000 zugreifbar ist.
 *      Wird dazu resetet, und ein Registertest wird durchgefuehrt.
 *      Return:
 *              0 = Registertest fehlgeschlagen
 *              1 = OK
 *      */
int     hwpelican_isinstalled(TCANDriver *CANDriver)
{
        int     i, n ;
#ifdef DEBUGOUT
sprintf(diagmsg, "Entering 'hwpelican_isinstalled()'...") ; putdiagmsg() ;
#endif
        HARDRESET ;
        REGWRITE(REG_MODE, REGREAD(REG_MODE) | 0x01 ) ; /* set RESET */
#ifdef DEBUGOUT
if ( REGREAD(REG_MODE) & 0x01 != 0x01 ) {
sprintf(diagmsg, "Resetbit im Moderegister nicht gesetzt!") ;
putdiagmsg() ;
}
#endif
        // set PeliCAN-Mode, leave Clockout und CBP as set by HARDRESET
        REGWRITE(REG_CLOCKDIV, REGREAD(REG_CLOCKDIV) | 0x80 ) ;
#ifdef DEBUGOUT
if ( REGREAD(REG_CLOCKDIV) & 0x80 != 0x80 ) {
sprintf(diagmsg, "Chip nicht im PeliCAN-Mode!") ;
putdiagmsg() ;
}
sprintf(diagmsg, "Registertest 1 ...") ; putdiagmsg() ;
         // Register 6..8,13,16..23 sind 8 Bit read/write
         // (Register 14,15 werden durch Bus-Off veraendert!)
#endif
        for (i=6 ; i <= 23 ; i++) if ( i <= 8 || i == 13 || i >= 16) {
                REGWRITE(i, i+5) ; // jedes Register mit 'Adresse+5' testen
                n = REGREAD(i) ;
                if ( n != i+5 ) {
#ifdef DEBUGOUT
sprintf(diagmsg, "Fehler bei Registertest 1: Reg[%d] ist 0x%x, soll sein 0x%x", i, n, i+5) ;
                        putdiagmsg() ;
#endif
                        return 0 ;
                }
        }
#ifdef DEBUGOUT
sprintf(diagmsg, "Registertest 2 ...") ; putdiagmsg() ;
#endif
        for (i=6 ; i <= 23 ; i++) if ( i <= 8 || i == 13 || i >= 16) {
                n = REGREAD(i) ;
                if ( n != i+5 ) {
#ifdef DEBUGOUT
sprintf(diagmsg, "Fehler bei Registertest 2: Reg[%d] ist 0x%x, soll sein 0x%x", i, n, i+5) ;
                        putdiagmsg() ;
#endif
                        return 0 ;
                }
        }
        return 1 ;
}


/**********************************************************************
 *      activate()
 *      Controller, wenn moeglich, hardresetten,
 *      neu programmieren, und aktiv schalten.
 *      */
void    hwpelican_activate(TCANDriver *CANDriver,
        unsigned short int btr0btr1,
        unsigned long int acc_code, // code+mask auch 32Bit, wie ID!
        unsigned long int acc_mask )
{

#ifdef XXX
        HARDRESET ;

        // reset
        REGWRITE(REG_MODE, REGREAD(REG_MODE) | 0x01 ) ;

        // set PeliCAN-Mode, leave Clockout und CBP as set by HARDRESET
        REGWRITE(REG_CLOCKDIV, REGREAD(REG_CLOCKDIV) | 0x80 ) ;
#else
// zum testen: Registertest bei Application-Hardwarereset
hwpelican_isinstalled(CANDriver) ;

#endif
        // no sleep (MOD.4==0),
        // vorlaeufig single acceptance filter (MOD.3==1),
        // no self test mode (MOD.2=0)
        // no listen only(MOD.1==0)
        // reset (MOD.0==1)
        REGWRITE(REG_MODE, 0x09) ;


        //  Clr Overrun, Release Rcvbuf, Abort Xmission
        REGWRITE(REG_COMMAND, 0x0E ) ;

        // Acceptance Code = Kennung fuer Empfang.
        // gueltig sind Bits 28..0, diese muessen auf
        // ACR/AMR0..3 verteilt werden.
        // keine Unterstuetzung des RTR-Bits, keine Unterstuetzung
        // der data bytes 0 und 1 bei standard frames-filterung
        // -> wird in registerMessages() unterdrueckt.
//sprintf(diagmsg, "acc_code=%x, acc_mask=%x", acc_code, acc_mask) ; putdiagmsg() ;
        REGWRITE(REG_ACCCODE0,  (acc_code >> 21) & 0xFF) ;
        REGWRITE(REG_ACCCODE1,  (acc_code >> 13) & 0xFF) ;
        REGWRITE(REG_ACCCODE2,  (acc_code >>  5) & 0xFF) ;
        REGWRITE(REG_ACCCODE3, ((acc_code << 3) & 0xFF) | 0x07) ;
        REGWRITE(REG_ACCMASK0,  (acc_mask >> 21) & 0xFF) ;
        REGWRITE(REG_ACCMASK1,  (acc_mask >> 13) & 0xFF) ;
        REGWRITE(REG_ACCMASK2,  (acc_mask >>  5) & 0xFF) ;
        REGWRITE(REG_ACCMASK3, ((acc_mask << 3) & 0xFF) | 0x07) ;

        /* Baudratenregister setzen */
        REGWRITE(REG_TIMING0, (btr0btr1 >> 8) & 0xFF ) ;
        REGWRITE(REG_TIMING1, btr0btr1 & 0xFF) ;

        /* Output Control setzen */
        REGWRITE(REG_OUTPUTCTRL, OCR_VALUE) ;

        /* Clear Interruptflags */
        REGREAD(REG_INTERRUPT) ;

        // Enable all Interrupts
        REGWRITE(REG_IRQENABLE, 0xFF) ;

        /* Reset Ende */
        REGWRITE(REG_MODE, 0x08) ;
//wg. err-test     REGWRITE(REG_MODE, REGREAD(REG_MODE) & ~0x01 ) ;
}


/************************************************************************
 *      isactive()
 *      1, wenn der Controller am Bus ist, sonst 0.
 *      */
int     hwpelican_isactive(TCANDriver *CANDriver)
{
        /* Ist am Bus, wenn das Statusbit 'Busstatus' nicht gesetzt ist */
        return ! (REGREAD(REG_STATUS) & 0x80) ;
}


/**************************************************************************
 *      disable()
 *      SJA1000/PeliCAN vom Bus nehmen, und reset
 *      */
void    hwpelican_disable(TCANDriver *CANDriver)
{
        REGWRITE(REG_MODE, REGREAD(REG_MODE) | 0x01 ) ;
}


/**********************************************************************
 *      intack()
 *      liest das Interuptregister des SJA1000/PeliCAN aus, gibt den
 *      Grund fuer den Interrupt zurueck, und loescht die Interruptbedingung.
 *      */
unsigned short int hwpelican_intack(TCANDriver *CANDriver)
{
        unsigned short int tmp ;
        /* Die Bits fuer Interrupt-Gruende (HW_IRQ_*)
         * sind passend fuer den 82C200.
         * Der Bus Error interrupt, der Errorpassive interrupt
         * und der Error Warning interrupt werden alle
         * auf HW_IRQ_ERROR gelegt. */
        tmp = REGREAD(REG_INTERRUPT)  ;  // Interruptregister lesen und loeschen
        if ( tmp & 0x80 ) tmp |= HW_IRQ_ERROR ; // Bus error interrupt
        if ( tmp & 0x20 ) tmp |= HW_IRQ_ERROR ; // Error passive interrupt
        return (tmp & 0x1f) ; // emuliere 82C200
}


/**********************************************************************
 *      msgread()
 *      Standard oder extended-Message aus dem Empfangsregister auslesen.
 *
 *      */
void hwpelican_msgread(TCANDriver *CANDriver,
        unsigned char *msgtype, unsigned long *id,
        unsigned char *len, unsigned char *data, TCANTimestamp *rcvtime)
{
        int     i, n ;
        int     frameinfo ;

        frameinfo = REGREAD(REG_FRAMEINFO) ;
        *msgtype =
                   ( frameinfo & 0x80 ? MSGTYPE_EXTENDED : 0)
                 | ( frameinfo & 0x40 ? MSGTYPE_RTR : 0) ;
        n = *len = frameinfo & 0x0f ;
        if ( frameinfo & 0x80 ) {
                unsigned long tmp ;
                /*** extended frame ***/
                tmp =   ( (unsigned long) REGREAD(REG_EFFID1) << 24)
                      | ( (unsigned long) REGREAD(REG_EFFID2) << 16)
                      | ( (unsigned long) REGREAD(REG_EFFID3) <<  8)
                      | ( (unsigned long) REGREAD(REG_EFFID4)      )  ;
                *id = tmp >> 3 ;
                if ( ! (frameinfo & 0x40) ) {
                         /* kein RTR: den ganzen Empfangspuffer nach 'data[]' bewegen */
                         if ( n > 8 ) n = 8 ; /* Laenge ggf. korrigieren */
                         for (i=0 ; i < n ; i++)
                                 data[i] = REGREAD(REG_EFFDATA+i) ;
                 }
         } else {
                /*** standard frame, wie 82C200 ***/
                *id = ((uword) REGREAD(REG_SFFID1) << 3) | (REGREAD(REG_SFFID2) >> 5) ;

                if ( ! (frameinfo & 0x40) ) {
                         /* kein RTR: den ganzen Empfangspuffer nach 'data[]' bewegen */
                         if ( n > 8 ) n = 8 ; /* Laenge ggf. korrigieren */
                         for (i=0 ; i < n ; i++)
                                 data[i] = REGREAD(REG_SFFDATA+i) ;
                 }
         }
//sprintf(diagmsg, "pelican-rcv: frameinfo=%x, msgtype=%x, len=%d, id=%x",
//frameinfo, *msgtype, *len, *id) ; putdiagmsg() ;
         /* release receive buffer */
         REGWRITE(REG_COMMAND, 0x04) ;
}


/**********************************************************************
 *      xmtempty()
 *      0, wenn keine Message an den Controller geschrieben werden darf.
 *
 *      Transmit buffer ist empty, wenn das 'transmit buffer access' Bit
 *      gesetzt ist
 *      */
int     hwpelican_xmtempty(TCANDriver *CANDriver)
{
        return  ( REGREAD(REG_STATUS) & 0x04 ) ;
}


/**********************************************************************
 *      msgwrite()
 *      Message in das Transmitregister schreiben.
 *      Kann Standard und Extended senden: immer return 1
 *      */
int hwpelican_msgwrite(TCANDriver *CANDriver,
        unsigned char msgtype, unsigned long id,
        unsigned char len, unsigned char *data)
{
        int     tmp ;
        int     i ;

        tmp =  (msgtype & MSGTYPE_EXTENDED ? 0x80 : 0)
           | (msgtype & MSGTYPE_RTR        ? 0x40 : 0)
           | (len & 0x0f) ;
        REGWRITE(REG_FRAMEINFO, tmp) ;
// sprintf(diagmsg, "pelican-xmt: frameinfo=%x, msgtype=%x, len=%d, id=%x",
// tmp, msgtype, len, id) ; putdiagmsg() ;

        if (msgtype & MSGTYPE_EXTENDED) {
                /*** extended frame ***/
                REGWRITE(REG_EFFID1, id >> 21) ;
                REGWRITE(REG_EFFID2, id >> 13) ;
                REGWRITE(REG_EFFID3, id >> 5) ;
                tmp = (id & 0x1F) << 3 ;
                if ( msgtype & MSGTYPE_RTR )
                        /* setze sicherheitshalber hier noch remote-request */
                        REGWRITE(REG_EFFID4, tmp | 0x04 ) ;
                else {
                        /* sende Datenmessage */
                        REGWRITE(REG_EFFID4, tmp ) ;

                        /* den ganzen Sendepuffer in den Controller bewegen */
                        if ( len > 8) len = 8 ;
                        for (i=0 ; i < len ; i++)
                                REGWRITE(REG_EFFDATA+i, data[i]) ;
               }
          } else {
                /*** standard frame ***/
                REGWRITE(REG_SFFID1, (uchar)(id >> 3) ) ;
                tmp = ((uchar) id & 0x07) << 5 ;
                if ( msgtype & MSGTYPE_RTR )
                        /* setze sicherheitshalber hier noch remote-request */
                        REGWRITE(REG_SFFID2, tmp | 0x10 ) ;
                else {
                        /* sende Datenmessage */
                        REGWRITE(REG_SFFID2, tmp ) ;

                        /* den ganzen Sendepuffer in den Controller bewegen */
                        if ( len > 8) len = 8 ;
                        for (i=0 ; i < len ; i++)
                                REGWRITE(REG_SFFDATA+i, data[i]) ;
               }
        }
         /* transmit data by setting of TRANSMISSION REQUEST */
        REGWRITE(REG_COMMAND, 0x01) ;
        return 1 ;
}

/************************************************************************
 *      buserror()
 *      setzt einen im Resetzustand stehenden SJA1000/PeliCAN wieder in Betrieb.
 *      Rueckgabe:
 *              0: alles war OK
 *              sonst: Fehlercode
 *      */
int hwpelican_buserror(TCANDriver *CANDriver)
{
        int     err ;

        err = 0 ;
        if ( REGREAD(REG_STATUS) & 0x80 ) {
            /* Bus-Off: Controller is NOT involved in bus activities */
            err |= CAN_ERR_BUSOFF ;
            /* Reset request auf 'absent' = Bit 0 loeschen */
            REGWRITE(REG_MODE, REGREAD(REG_MODE) & ~0x01) ;
            /* Der Controller geht wieder 'On-Bus' nach dem Empfang
             * von 128 * 11 aufeinanderfolgenden rezessiven Bits.
             */
        } else {
            /* Bus-On: Controller is involved in bus activities */
            if ( REGREAD(REG_STATUS) & 0x40 )
                /* schwere Stoerung: ein Errorcounter erreichte Limit */
                err |= CAN_ERR_BUSHEAVY ;
       }
        if ( REGREAD(REG_STATUS) & 0x02 )
                err |= CAN_ERR_OVERRUN ;

        return  err ;
}


/************************************************************************
 *      clroverrun()
 *      Overrun-Bedingung loeschen
 *      */
void    hwpelican_clroverrun(TCANDriver *CANDriver)
{
        REGWRITE(REG_COMMAND, 0x08) ;
}





/***********************************************************************
 *      regname()
 *      gibt den Klartextnamen einer Registeradresse zurueck.
 *      */
static char * regname(int reg) {
        switch ( reg ) {
        case REG_MODE       : return "MODE" ;
        case REG_COMMAND    : return "COMMAND" ;
        case REG_STATUS     : return "STATUS" ;
        case REG_INTERRUPT  : return "INTERRUPT" ;
        case REG_TIMING0    : return "TIMING0" ;
        case REG_TIMING1    : return "TIMING1" ;
        case REG_OUTPUTCTRL : return "OUTPUTCTRL" ;

        default          : return "UNKNOWN" ;
        }
}


